require 'installer/action'
require 'installer/errors'

class Installer::Planner::Step::CompatibilityCheck < Installer::Planner::Step
  def initialize(infra, data={}, pre=nil, post=nil)
    super
  end

  def perform_step(&execute)
    os_data = Hash.new {|h,k| h[k] = {}}

    # Identify the Installer being used
    packages_dir = "/opt/puppet/share/installer/installer/packages"
    available_platforms = Dir.glob("#{packages_dir}/*").select { |f| File.directory?(f) }.map { |f| File.basename(f) }

    @infra.hosts.each do |hostname, host|
      # Ensure PE is not already installed
      peinstalled = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "ls /opt/puppet/bin/puppet")
      yield peinstalled

      if peinstalled.success?
        peversion = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "cat /opt/puppet/pe_version")
        yield peversion

        if peversion.success?
          error(hostname, "Puppet Enterprise #{peversion.results[hostname][:stdout].strip} is already installed.")
        else
          error(hostname, "An unknown version of Puppet Enterprise is already installed.")
        end
      end

      # First try identifying using lsb_release.  This takes care of Ubuntu (lsb-release is part of ubuntu-minimal).
      # Original bash TODO: Remove this junk
      #   if type lsb_release > /dev/null 2>&1; then
      # t_prepare_platform=`lsb_release -icr 2>&1`
      #
      # PLATFORM_NAME="$(printf "${t_prepare_platform?}" | ${PLATFORM_EGREP?} '^Distributor ID:' | cut -s -d: -f2 | sed 's/[[:space:]]//' | tr '[[:upper:]]' '[[:lower:]]')"
      #
      # # Sanitize name for unusual platforms
      # case "${PLATFORM_NAME?}" in
      #     redhatenterpriseserver | redhatenterpriseclient | redhatenterpriseas | redhatenterprisees | enterpriseenterpriseserver | redhatenterpriseworkstation | redhatenterprisecomputenode | oracleserver)
      #         PLATFORM_NAME=rhel
      #         ;;
      #     enterprise* )
      #         PLATFORM_NAME=centos
      #         ;;
      #     scientific | scientifics | scientificsl )
      #         PLATFORM_NAME=rhel
      #         ;;
      #     'suse linux' )
      #         PLATFORM_NAME=sles
      #         ;;
      #     amazonami )
      #         PLATFORM_NAME=amazon
      #         ;;
      # esac
      #
      # # Release
      # PLATFORM_RELEASE="$(printf "${t_prepare_platform?}" | ${PLATFORM_EGREP?} '^Release:' | cut -s -d: -f2 | sed 's/[[:space:]]//g')"
      #
      # # Sanitize release for unusual platforms
      # case "${PLATFORM_NAME?}" in
      #     centos | rhel )
      #         # Platform uses only number before period as the release, e.g. "CentOS 5.5" is release "5"
      #         PLATFORM_RELEASE="$(printf "${PLATFORM_RELEASE?}" | cut -d. -f1)"
      #         ;;
      #     debian )
      #         # Platform uses only number before period as the release, e.g. "Debian 6.0.1" is release "6"
      #         PLATFORM_RELEASE="$(printf "${PLATFORM_RELEASE?}" | cut -d. -f1)"
      #         if [ ${PLATFORM_RELEASE} = "testing" ] ; then
      #             PLATFORM_RELEASE=7
      #         fi
      #         ;;
      # esac
      lsb = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "lsb_release -icr")
      yield lsb

      if lsb.success?
        lsb_data = {}
        lsb.results[hostname][:stdout].each_line do |l|
          k, v = l.split(':')
          if k && v
            lsb_data[k.strip] = v.strip
          end
        end

        distribution = lsb_data['Distributor ID'].downcase.gsub(/\s+/, "")
        distribution = case distribution
        when /redhatenterpriseserver|redhatenterpriseclient|redhatenterpriseas|redhatenterprisees|enterpriseenterpriseserver|redhatenterpriseworkstation|redhatenterprisecomputenode|oracleserver/
          :rhel
        when /enterprise.*/
          :centos
        when /scientific|scientifics|scientificsl/
          :rhel
        when /amazonami/
          :amazon
        when /suselinux/
          :sles
        else
          distribution.to_sym
        end

        release = lsb_data['Release'].gsub(/\s+/, "")
        release = case distribution
        when :centos, :rhel
          release.split('.')[0]
        when :debian
          if release == 'testing'
            '7'
          else
            release.split('.')[0]
          end
        else
          release
        end

      end


      # Original bash TODO: Remove this junk
      # elif [ -f /etc/redhat-release -a -r /etc/redhat-release -a -s /etc/redhat-release ]; then
      #     # Oracle Enterprise Linux 5.3 and higher identify the same as RHEL
      #     if grep -qi 'red hat enterprise' /etc/redhat-release; then
      #         PLATFORM_NAME=rhel
      #     elif grep -qi 'centos' /etc/redhat-release; then
      #         PLATFORM_NAME=centos
      #     elif grep -qi 'scientific' /etc/redhat-release; then
      #         PLATFORM_NAME=rhel
      #     fi
      #     # Release - take first digit after ' release ' only.
      #     PLATFORM_RELEASE="$(sed 's/.*\ release\ \([[:digit:]]\).*/\1/g;q' /etc/redhat-release)"
      if !distribution && !release
        redhat_release = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "cat /etc/redhat-release")
        yield redhat_release

        if redhat_release.success?
          stdout = redhat_release.results[hostname][:stdout]

          distribution = case stdout
          when /red hat enterprise/im
            :rhel
          when /centos/im
            :centos
          when /scientific/im
            :rhel
          end

          release = /.* release ([[:digit:]]).*/.match(stdout)[1]
        end
      end

      # Original bash TODO: Remove this junk
      # elif [ -r "/etc/os-release" ] && ${PLATFORM_EGREP?} "Cumulus Linux" "/etc/os-release" &> /dev/null ; then
      #       PLATFORM_NAME=cumulus
      #       PLATFORM_RELEASE=`${PLATFORM_EGREP?} "VERSION_ID" "/etc/os-release" | cut -d'=' -f2 | cut -d'.' -f'1,2'`
      if !distribution && !release
        os_release = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "cat /etc/os-release")
        yield os_release

        if os_release.success?
          stdout = os_release.results[hostname][:stdout]

          if /Cumulus Linux/m.match(stdout)
            distribution = :cumulus
          end

          release = /VERSION_ID=(\d+\.\d)/.match(stdout)[1]
        end
      end

      #   elif [ -r "/etc/Eos-release" ] && ${PLATFORM_EGREP?} "Arista Networks EOS" "/etc/Eos-release" &> /dev/null ; then
      #       PLATFORM_NAME=eos
      #       PLATFORM_RELEASE=$(sed -n 's/^Arista Networks EOS v*\(.*\)$/\1/p' "/etc/Eos-release" | cut -d'.' -f1)
      if !distribution && !release
        eos_release = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "cat /etc/Eos-release")
        yield eos_release

        if eos_release.success?
          stdout = eos_release.results[hostname][:stdout]

          if /Arista Networks EOS/m.match(stdout)
            distribution = :eos
          end

          release = /^Arista Networks EOS v*(.*)\..*$/.match(stdout)[1]
        end
      end

      #   # Test for Debian releases
      #   elif [ -f /etc/debian_version -a -r /etc/debian_version -a -s /etc/debian_version ]; then
      #       t_prepare_platform__debian_version_file="/etc/debian_version"
      #       t_prepare_platform__debian_version=`cat /etc/debian_version`
      #
      #       if cat "${t_prepare_platform__debian_version_file?}" | ${PLATFORM_EGREP?} '^[[:digit:]]' > /dev/null; then
      #           PLATFORM_NAME=debian
      #           PLATFORM_RELEASE="$(printf "${t_prepare_platform__debian_version?}" | sed 's/\..*//')"
      #       elif cat "${t_prepare_platform__debian_version_file?}" | ${PLATFORM_EGREP?} '^wheezy' > /dev/null; then
      #           PLATFORM_NAME=debian
      #           PLATFORM_RELEASE="7"
      #       fi
      if !distribution && !release
        debian_version = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "cat /etc/debian_version")
        yield debian_version

        if debian_version.success?
          stdout = debian_version.results[hostname][:stdout]
          distribution = :debian

          release = case stdout
          when /^[[:digit:]]/
            stdout.split('.')[0]
          when /^wheezy/.match(stdout)
            "7"
          end
        end
      end

      #   elif [ -f /etc/SuSE-release -a -r /etc/SuSE-release ]; then
      #       t_prepare_platform__suse_version=`cat /etc/SuSE-release`
      #
      #       if printf "${t_prepare_platform__suse_version?}" | ${PLATFORM_EGREP?} 'Enterprise Server'; then
      #           PLATFORM_NAME=sles
      #           t_version=`/bin/cat /etc/SuSE-release | grep VERSION | sed 's/^VERSION = \(\d*\)/\1/' `
      #           t_patchlevel=`cat /etc/SuSE-release | grep PATCHLEVEL | sed 's/^PATCHLEVEL = \(\d*\)/\1/' `
      #           PLATFORM_RELEASE="${t_version}"
      #       fi
      if !distribution && !release
        suse_release = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "cat /etc/SuSE-release")
        yield suse_release

        if suse_release.success?
          stdout = suse_release.results[hostname][:stdout]

          if /Enterprise Server/.match(stdout)
            distribution = :sles
            release = /^VERSION = (\d*)/m.match(stdout)[1]
          end
        end
      end

      #   elif [ -f /etc/system-release ]; then
      #       if grep -qi 'amazon linux' /etc/system-release; then
      #           PLATFORM_NAME=amazon
      #           PLATFORM_RELEASE=6
      #       else
      #           display_platform_failure "$(cat /etc/system-release) is not a supported platform for Puppet Enterprise v${PE_VERSION}
      #               Please visit http://links.puppetlabs.com/puppet_enterprise_${PE_LINK_VER?}_platform_support to request support for this platform."
      if !distribution && !release
        system_release = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "cat /etc/system-release")
        yield system_release

        if system_release.success?
          stdout = system_release.results[hostname][:stdout]

          if /amazon linux/im.match(stdout)
            distribution = :amazon
            # How is this safe to assume?
            release = "6"
          else
            error(host, "#{stdout} is not a supported platform for this version of Puppet Enterprise.")
            next
          end
        end
      end


      # See if it is Solaris or AIX
      # Original bash TODO: Remove this junk
      #       elif [ "x$(uname -s)" = "xSunOS" ]; then
      #     PLATFORM_NAME="solaris"
      #     t_platform_release="$(uname -r)"
      #     # JJM We get back 5.10 but we only care about the right side of the decimal.
      #     PLATFORM_RELEASE="${t_platform_release##*.}"
      #     PLATFORM_EGREP='egrep'
      # elif [ "x$(uname -s)" = "xAIX" ] ; then
      #     PLATFORM_NAME="aix"
      #     t_platform_release="$(oslevel | cut -d'.' -f1,2)"
      #     PLATFORM_RELEASE="${t_platform_release}"
      #     PLATFORM_EGREP='egrep'
      #     PLATFORM_ROOT_GROUP='system'
      #     PLATFORM_ROOT_USER='root'
      #     PLATFORM_PUPPET_GROUP='puppet'
      #     PLATFORM_PUPPET_USER='puppet'
      #     PLATFORM_SYMLINK_TARGET='/usr/bin'
      #     PLATFORM_PUPPET_HOME='/opt/freeware/var/lib/pe-puppet'
      if !distribution && !release
        uname = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "uname -s")
        yield uname

        if uname.success?
          distribution = case uname.results[hostname][:stdout].strip
          when "SunOS"
            unamer = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "uname -r")
            yield unamer

            if unamer.success?
              release = unamer.results[hostname][:stdout].split('.')[0]
            end

            :solaris
          when "AIX"
            oslevel = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "oslevel")
            yield oslevel

            if oslevel.success?
              release = oslevel.results[hostname][:stdout].split('.')[0..1].join('.')
            end

            :aix
          end
        end
      end

      # If we haven't found the distribution or release by this point, it's definitely not supported.
      unless distribution && release
        error(hostname, "Puppet Enterprise could not detect the distribution and release.")
        next
      end

      # Bash from puppet-enterprise-installer
      # Architecture
      # if [ -z "${PLATFORM_ARCHITECTURE:-""}" ]; then
      #     case "${PLATFORM_NAME?}" in
      #         solaris | aix )
      #             PLATFORM_ARCHITECTURE="$(uname -p)"
      #             if [ "${PLATFORM_ARCHITECTURE}" = "powerpc" ] ; then
      #               PLATFORM_ARCHITECTURE='power'
      #             fi
      #             ;;
      #         *)
      #             PLATFORM_ARCHITECTURE="`uname -m`"
      #             ;;
      #     esac
      #     case "${PLATFORM_ARCHITECTURE?}" in
      #         x86_64)
      #             case "${PLATFORM_NAME?}" in
      #                 ubuntu | debian )
      #                     PLATFORM_ARCHITECTURE=amd64
      #                     ;;
      #             esac
      #             ;;
      #         i686)
      #             PLATFORM_ARCHITECTURE=i386
      #             ;;
      #         ppc)
      #             PLATFORM_ARCHITECTURE=powerpc
      #             ;;
      #     esac
      # fi
      unamem = Installer::Action::Execute.new(:host => host, :sudo => true, :cmd => "uname -m")
      yield unamem

      if unamem.success?
        architecture = unamem.results[hostname][:stdout].strip

        architecture = case architecture
        when 'i686'
          'i386'
        when 'ppc'
          'powerpc'
        when 'x86_64'
          [:ubuntu, :debian].include?(distribution) ? 'amd64' : 'x86_64'
        else
          architecture
        end
      end

      unless architecture
        error(hostname, "Puppet Enterprise could not detect the architecture.")
        next
      end

      # Tag
      # if [ -z "${PLATFORM_TAG:-""}" ]; then
      #     case "${PLATFORM_NAME?}" in
      #         # Enterprise linux (centos & rhel) share the same packaging
      #         # Amazon linux is similar enough for our packages
      #         rhel | centos | amazon )
      #             PLATFORM_TAG="el-${PLATFORM_RELEASE?}-${PLATFORM_ARCHITECTURE?}"
      #             ;;
      #         *)
      #             PLATFORM_TAG="${PLATFORM_NAME?}-${PLATFORM_RELEASE?}-${PLATFORM_ARCHITECTURE?}"
      #             ;;
      #     esac
      # fi
      os_data[hostname][:distribution] = distribution
      os_data[hostname][:release] = release
      os_data[hostname][:architecture] = architecture
      os_data[hostname][:tag] = case distribution
      when :rhel, :centos, :amazon
        "el-#{release}-#{architecture}"
      else
        "#{distribution}-#{release}-#{architecture}"
      end
    end

    platform_tags = os_data.values.collect { |d| d[:tag] }.uniq

    if platform_tags.size != 1
      error("localhost", "Multiple server platforms detected: #{platform_tags.join(', ')}. Installing across different server platforms is not supported.")
    end

    platform_tag = platform_tags.first

    if ! available_platforms.include? platform_tag.to_s
      error("localhost", "Could not find #{platform_tag} installer from available installers: #{available_platforms.join(', ')}.")
    end

    os_data
  end

  def name
    :compatibility_check
  end

  def description
    "Verifying that #{@infra.installer_hostname} has a PE installer that matches #{@infra.hosts.keys.join(', ')}'s OS."
  end
end
